# Fig. 19.8: SynchronizedInteger.py
# Synchronized access to an integer with condition variable
import threading
class SynchronizedInteger:
	"""Class that provides synchronized access an integer"""
	def __init__( self ):
	   """Set shared number, write flag and condition variable"""
	   self.sharedNumber = -1
	   self.writeable = 1      # the value can be changed
	   self.threadCondition = threading.Condition()
	def setSharedNumber( self, newNumber ):
	   """Set value of integer--blocks until lock acquired"""
	   # block until lock released then acquire lock
	   self.threadCondition.acquire()
	   # while not producer's turn, release lock and block
	   while not self.writeable:
	      self.threadCondition.wait()
	   # (lock has now been re-acquired)
	   print "%s setting sharedNumber to %d" % \
	      ( threading.currentThread().getName(), newNumber )
	   self.sharedNumber = newNumber
	   self.writeable = 0             # allow consumer to consume
	   self.threadCondition.notify() # wake up a waiting thread
	   self.threadCondition.release() # allow lock to be acquired
	def getSharedNumber( self ):
	   """Get value of integer--blocks until lock acquired"""
	   # block until lock released then acquire lock
	   self.threadCondition.acquire()
	   # while producer's turn, release lock and block
	   while self.writeable:
	      self.threadCondition.wait()
	   # (lock has now been re-acquired)
	   tempNumber = self.sharedNumber
	   print "%s retrieving sharedNumber value %d" % \
	      ( threading.currentThread().getName(), tempNumber )
	   self.writeable = 1             # allow producer to produce
	   self.threadCondition.notify() # wake up a waiting thread
	   self.threadCondition.release() # allow lock to be acquired
	   return tempNumber

